﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Microsoft.Phone.Controls;
using System.Threading;
using QtsHttp;
using QtsHttp.Comm;
using QtsHttp.FileStaion;

namespace PhoneApp1_7._5_Sample
{
    public partial class MainPage : PhoneApplicationPage
    {
        private Server _qtsServer = null;
        private IQtsHttpFileStation _fsObj = null;
        
        // Constructor
        public MainPage()
        {
            InitializeComponent();
        }

        private void WatermarkTBGotFocus(object sender, RoutedEventArgs e)
        {
            if (hostName.Text == "255.255.255.255")
            {
                SolidColorBrush Brush1 = new SolidColorBrush();
                Brush1.Color = Colors.Black;

                hostName.Text = "";
                hostName.Foreground = Brush1;

                port.Text = "";
                port.Foreground = Brush1;

                userName.Text = "";
                userName.Foreground = Brush1;

                password.Password = "";
                password.Foreground = Brush1;

            }
        }

        private void WatermarkTBLostFocus(object sender, RoutedEventArgs e)
        {
        }

        async private void ButtonClickLogin(object sender, RoutedEventArgs e)
        {   
            if (hostName.Text == "" || IsIPv4(hostName.Text) == false)
            {
                MessageBox.Show("Invalid Host Name!");
                return;
            }
            if (userName.Text == string.Empty)
            {
                MessageBox.Show("Please input User Name!");
                return;
            }
            if (password.Password == string.Empty)
            {
                MessageBox.Show("Please input Password!");
                return;
            }

            string serHostName = hostName.Text;
            string serUserName = userName.Text;
            string serPassword = password.Password;
            string strComputerName = string.Empty;
            long serPort = Convert.ToInt64(port.Text);

            ServerInfo serverInfo = new ServerInfo(serHostName, serUserName, serPassword, false, strComputerName);
            _qtsServer = new Server(serverInfo);
            _qtsServer.SetFileStationPortNum(Convert.ToInt64(port.Text));
            try
            {
                await _qtsServer.Login(StationType.FileStation);

                if (_qtsServer.QueryFileStationVersion() == APIVersion.FileStationApiV1)
                {
                    _fsObj = _qtsServer.OpenFileStation();
                    logoutButton.IsEnabled = true;
                    loginButton.IsEnabled = false;
                    NavigationService.Navigate(new Uri("/Page1.xaml", UriKind.RelativeOrAbsolute));
                }

            }
            catch (Exception ex)
            {
                MessageBox.Show(serHostName + " Login Failed! " + ex.Message, "Login Message", MessageBoxButton.OK);
            }
            
        }

        async private void ButtonClickLogout(object sender, RoutedEventArgs e)
        {
            string serHostName = hostName.Text;

            if (_fsObj == null)
            {
                MessageBox.Show("Please Login before Logout.");
                return;
            }
            else
            {
                try
                {
                    await _qtsServer.CloseFileStation(_fsObj);
                    logoutButton.IsEnabled = false;
                    loginButton.IsEnabled = true;
                }
                catch (Exception ex)
                {
                    MessageBox.Show(serHostName + " Logout Failed! " + ex.Message, "Logout Message", MessageBoxButton.OK);
                }
            }
        }

        public bool IsIPv4(string value)
        {
            var quads = value.Split('.');

            // if we do not have 4 quads, return false
            if (!(quads.Length == 4)) return false;

            // for each quad
            foreach (var quad in quads)
            {
                int q;
                // if parse fails 
                // or length of parsed int != length of quad string (i.e.; '1' vs '001')
                // or parsed int < 0
                // or parsed int > 255
                // return false
                if (!Int32.TryParse(quad, out q)
                    || !q.ToString().Length.Equals(quad.Length)
                    || q < 0
                    || q > 255) { return false; }
            }

            return true;
        }

        protected override void OnNavigatedFrom(System.Windows.Navigation.NavigationEventArgs e)
        {
            App app = Application.Current as App;
            app.gFSObj = _fsObj;
            base.OnNavigatedFrom(e);
        }
    }
}