﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Navigation;
using Microsoft.Phone.Controls;
using Microsoft.Phone.Shell;
using System.Threading;
using System.IO.IsolatedStorage;
using System.IO;
using System.Threading.Tasks;
using System.Windows.Threading;
using System.Collections.ObjectModel;
using QtsHttp.Comm;
using QtsHttp.FileStaion;

namespace PhoneApp1_7._5_Sample
{
    public partial class Page_Download : PhoneApplicationPage
    {
        App app = Application.Current as App;
        ObservableCollection<FileEntry> collection = new ObservableCollection<FileEntry>();
        List<FileEntry> returnFileList = new List<FileEntry>();
        string previousPath = string.Empty;
        string dowloadInputName = string.Empty;
        CancellationTokenSource reqCancel = new CancellationTokenSource();
        DispatcherTimer timer;

        public Page_Download()
        {

            this.Loaded += Page_Download_Loaded;
            InitializeComponent();
            FileListBox.ItemsSource = collection;
            
            InitDisplay();
        }

        void Page_Download_Loaded(object sender, RoutedEventArgs e)
        {
            DownloadProgress.Maximum = 100;
        }

        async private void InitDisplay()
        {
            try
            {
                returnFileList = await app.gFSObj.GetShareFolderList(reqCancel);
            }
            catch (Exception ex)
            {
                MessageBox.Show("Get Share Folder Failed! " + ex.Message, "Get Share Folder Message", MessageBoxButton.OK);
            }

            foreach (FileEntry iter in returnFileList)
            {
                collection.Add(iter);
            }

            returnFileList.Clear();
        }

        async private void OnSelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            ListBox source = (ListBox)sender;
            
            if (source.SelectedIndex == -1) // prevent execute this function twice at the same time
                return;

            if ((source.SelectedItem as FileEntry).IsDir != true)
            {
                return;
            }
            

            string currPath = previousPath + "/" + (source.SelectedItem as FileEntry).FileName;

            collection.Clear();
            int returnFileCount = 0;
            try
            {
                returnFileCount = await app.gFSObj.GetFileListCount(currPath, reqCancel);
            }
            catch (Exception ex)
            {
                MessageBox.Show("Get File List Count Failed! " + ex.Message, "Get File List Count Message", MessageBoxButton.OK);
            }

            try
            {
                returnFileList = await app.gFSObj.GetFileList(currPath, 0, returnFileCount, reqCancel);
            }
            catch (Exception ex)
            {
                MessageBox.Show("Get File List Failed! " + ex.Message, "Get File List Message", MessageBoxButton.OK);
            }

            foreach (FileEntry iter in returnFileList)
            {
                collection.Add(iter);
            }

            previousPath = currPath;

            source.SelectedIndex = -1;

            return;
        }

       
        private void ContextMenu_Opened(object sender, RoutedEventArgs e)
        {
            ContextMenu cm = sender as ContextMenu;
            FileEntry target = cm.DataContext as FileEntry;
            if (target.IsDir == true)
            {
                cm.IsOpen = false;
                return;
            }
        }

        private void menuDownload_Click(object sender, RoutedEventArgs e)
        {
            MenuItem source = (MenuItem)sender;
            FileEntry target = source.DataContext as FileEntry;
            if (target.IsDir == true)
            {
                return;
            }

            string composeSrcPath = target.FilePath + "/" + target.FileName;
            Download_Start(composeSrcPath);
        }

        
        async private void Download_Start(string srcpath)
        {
            string strSrcPath = srcpath;
            int lastSlash = strSrcPath.LastIndexOf("/");
            string splitFolderPath = strSrcPath.Substring(0, lastSlash);
            string splitFileName = strSrcPath.Substring(lastSlash + 1, (strSrcPath.Length - splitFolderPath.Length) - 1);

            FileEntry downloadInput = new FileEntry();
            downloadInput.FilePath = splitFolderPath;
            downloadInput.FileName = splitFileName;

            FileStatus returnFileStatus = new FileStatus();
            try
            {
                returnFileStatus = await app.gFSObj.GetFileStatus(downloadInput.FilePath + "/" + downloadInput.FileName, reqCancel);
                if (returnFileStatus.FileSize != 0)
                {
                    if (MessageBox.Show("Filename: " + returnFileStatus.FileName + " size: " + returnFileStatus.FileSize.ToString() + ", Download Start!", "Confirm message", MessageBoxButton.OKCancel) == MessageBoxResult.Cancel)
                    {
                        return;
                    }
                }
                else
                {
                    MessageBox.Show("File does not exist!");
                    return;
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }

            downloadInput.FileSize = returnFileStatus.FileSize;
            SyncProgressCallback progress = new SyncProgressCallback(DownloadUpload_Progress);

            Stream returnStream = null;
            try
            {
                returnStream = await app.gFSObj.DownloadFile(downloadInput, "\\Shared\\transfers\\", 0, this, progress, reqCancel);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
                return;
            }

            string localPath = "Shared";
            try
            {
                await SaveStream(downloadInput.FileName, localPath, returnStream, (ulong)downloadInput.FileSize, progress, reqCancel);
            }
            catch (OperationCanceledException ex)
            {
                string exception = ex.ToString();
                throw;
            }

        }


        private static async Task SaveStream(string filename, string parentfolder, Stream input, ulong dwloadSize, SyncProgressCallback HttpFSSyncProgressCallback, CancellationTokenSource FSCancellation)
        {
            ulong totalsize = 0;

            IsolatedStorageFile store = IsolatedStorageFile.GetUserStoreForApplication();
            // store file to Shared subfolder

            if (!store.GetDirectoryNames().Contains(parentfolder))
            {
                try
                {
                    store.CreateDirectory(parentfolder);
                }
                catch (Exception ex)
                {
                    MessageBox.Show("Can't create path: " + parentfolder);
                    return;
                }
            }

            //check filename if already exist
            filename = ComposeNewFileName(store, parentfolder, filename);

            string filepath = parentfolder + "/" + filename;
            byte[] buffer = new byte[5242880];
            ulong nTotalLength = dwloadSize;
            int ncount = 0;
            int nindex = 0;
            using (IsolatedStorageFileStream newfile = new IsolatedStorageFileStream(filepath, FileMode.OpenOrCreate, store))
            {
                
                while ((ncount = await input.ReadAsync(buffer, 0, buffer.Length)) > 0)
                {
                    if (FSCancellation.Token.IsCancellationRequested)
                    {
                        return;
                    }
                    
                    FSCancellation.Token.ThrowIfCancellationRequested();
                    totalsize += (ulong)ncount;
                    await newfile.WriteAsync(buffer, 0, (int)ncount);
                    int nprogress = (int)(totalsize * 100 / nTotalLength);
                    System.Diagnostics.Debug.WriteLine("progress: " + nprogress);
                    if (nindex % 10 == 0)
                    {
                        if (HttpFSSyncProgressCallback != null)
                        {
                            HttpFSSyncProgressCallback(totalsize, nprogress);
                        }
                        else
                        {
                            System.Diagnostics.Debug.WriteLine(string.Format("download progress {0}", nprogress));
                        }
                        await Delay(50);
                        
                    }
                    else if (totalsize == nTotalLength)
                    {
                        if (HttpFSSyncProgressCallback != null)
                        {
                            HttpFSSyncProgressCallback(totalsize, nprogress);
                        }
                        await Delay(50);
                    }
                
                    nindex++;
                    System.Diagnostics.Debug.WriteLine("Current: " + Microsoft.Phone.Info.DeviceStatus.ApplicationCurrentMemoryUsage.ToString());
                      //newfile.Flush(true);

                    await newfile.FlushAsync();
                } // end of while

                System.Diagnostics.Debug.WriteLine(string.Format("download size={0}", totalsize / 1024));

            } // end of using

        }

        private static string ComposeNewFileName(IsolatedStorageFile store, string filepath, string filename)
        {
            string newname = filename;
            int nSequent = 1;
            while (store.FileExists(filepath + "/" + newname))
            {
                string extension = System.IO.Path.GetExtension(newname);
                string newtitle = System.IO.Path.GetFileNameWithoutExtension(newname);
                int nbegin = newtitle.IndexOf("(");
                int nend = newtitle.IndexOf(")");
                if (nbegin > 0 && nend > 0 && nend > nbegin + 1)
                {
                    string strdig = newtitle.Substring(nbegin + 1, nend - nbegin - 1);
                    bool bisnumber = int.TryParse(strdig, out nSequent);
                    if (bisnumber)
                    {
                        newtitle = newtitle.Substring(0, nbegin);
                        nSequent++;
                    }
                }
                newname = string.Format("{0}({1}){2}", newtitle, nSequent, extension);
            }
            return newname;
        }
        
        void DownloadUpload_Progress(object sender, int nProgress)
        {
            app.gProgress = nProgress;
            DownloadProgress.Value = nProgress;
            if (nProgress == 100)
            {
                MessageBox.Show("Download Complete");
                DownloadProgress.Visibility = Visibility.Collapsed;
                DownloadProgress.Value = 0;
                DownloadProgress.Visibility = Visibility.Visible;
                NavigationService.Navigate(new Uri("/Page_ShowLocalFile.xaml", UriKind.RelativeOrAbsolute));
            }
        }

        protected override void OnNavigatedTo(System.Windows.Navigation.NavigationEventArgs e)
        {
            if (app.gFSObj == null)
            {
                MessageBox.Show("File Station Login Failed!");
                return;
            }
            base.OnNavigatedTo(e);
        }

        protected override void OnNavigatedFrom(System.Windows.Navigation.NavigationEventArgs e)
        {
            App app = Application.Current as App;
            base.OnNavigatedFrom(e);
        }

        async protected override void OnBackKeyPress(System.ComponentModel.CancelEventArgs e)
        {
            collection.Clear();
            returnFileList.Clear();

            int lastSlashPos = previousPath.LastIndexOf("/");

            if (lastSlashPos == -1) // under share_root, go back function menu
            {
                e.Cancel = false;
            }
            else if (lastSlashPos == 0) //under first level of share_root
            {
                e.Cancel = true;
                try
                {
                    returnFileList = await app.gFSObj.GetShareFolderList(reqCancel);
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message);
                }
                previousPath = string.Empty;
            }
            else
            {
                e.Cancel = true;
                previousPath = previousPath.Substring(0, lastSlashPos);
                lastSlashPos = previousPath.LastIndexOf("/");
                try
                {
                    int returnFileCount = await app.gFSObj.GetFileListCount(previousPath, reqCancel);
                    returnFileList = await app.gFSObj.GetFileList(previousPath, 0, returnFileCount, reqCancel);
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message);
                }
                //get title text
                string temp = previousPath;
                int tempLastSlashPos = temp.LastIndexOf("/");
                temp = temp.Remove(0, tempLastSlashPos);

            }

            FileListBox.SelectedIndex = -1;

            foreach (FileEntry iter in returnFileList)
            {
                collection.Add(iter);
            }

            FileListBox.ItemsSource = collection;
        }

        public static Task<DateTimeOffset> Delay(int millisecondsTimeout)
        {
            TaskCompletionSource<DateTimeOffset> tcs = null;
            Timer timer = null;

            timer = new Timer(delegate
            {
                timer.Dispose();
                tcs.TrySetResult(DateTimeOffset.UtcNow);
            }, null, Timeout.Infinite, Timeout.Infinite);

            tcs = new TaskCompletionSource<DateTimeOffset>(timer);
            timer.Change(millisecondsTimeout, Timeout.Infinite);
            return tcs.Task;
        }

    }
}